@@ -49,6 +49,8 @@ else:
         @classmethod
         async def transform(cls, interaction: InteractionT, argument: str) -> str:
             ctx = await interaction.client.get_context(interaction)
+            if interaction and not interaction.response.is_done():
+                await ctx.defer(ephemeral=True)
             return await cls.convert(ctx, argument)
 
         @classmethod
