@@ -14,11 +14,13 @@ from pylavcogs_shared.ui.buttons.generic import CloseButton, NavigateButton, Ref
 from pylavcogs_shared.ui.buttons.queue import (
     DecreaseVolumeButton,
     DisconnectButton,
+    EmptyQueueButton,
     EnqueueButton,
     IncreaseVolumeButton,
     PauseTrackButton,
     PlayNowFromQueueButton,
     PreviousTrackButton,
+    QueueHistoryButton,
     RemoveFromQueueButton,
     ResumeTrackButton,
     ShuffleButton,
@@ -31,6 +33,7 @@ from pylavcogs_shared.ui.menus.generic import BaseMenu
 from pylavcogs_shared.ui.selectors.playlist import PlaylistSelectSelector
 from pylavcogs_shared.ui.selectors.queue import EffectsSelector, QueueSelectTrack, SearchSelectTrack
 from pylavcogs_shared.ui.sources.queue import EffectsPickerSource, QueuePickerSource, QueueSource, SearchPickerSource
+from pylavcogs_shared.utils.decorators import is_dj_logic
 
 _ = Translator("PyLavShared", Path(__file__))
 
@@ -49,6 +52,7 @@ class QueueMenu(BaseMenu):
         timeout: int = 600,
         message: discord.Message = None,
         starting_page: int = 0,
+        history: bool = False,
         **kwargs: Any,
     ) -> None:
         super().__init__(
@@ -62,7 +66,7 @@ class QueueMenu(BaseMenu):
             **kwargs,
         )
         self.author = original_author
-
+        self.is_history = history
         self.forward_button = NavigateButton(
             style=discord.ButtonStyle.grey,
             emoji="\N{BLACK RIGHT-POINTING TRIANGLE}\N{VARIATION SELECTOR-16}",
@@ -117,12 +121,18 @@ class QueueMenu(BaseMenu):
             row=1,
             cog=cog,
         )
+        self.show_history_button = QueueHistoryButton(
+            style=discord.ButtonStyle.grey,
+            row=1,
+            cog=cog,
+        )
 
         self.close_button = CloseButton(
             style=discord.ButtonStyle.red,
             row=1,
             cog=cog,
         )
+        self.clear_queue_button = EmptyQueueButton(style=discord.ButtonStyle.red, row=1, cog=cog)
 
         self.previous_track_button = PreviousTrackButton(
             style=discord.ButtonStyle.grey,
@@ -190,8 +200,11 @@ class QueueMenu(BaseMenu):
     async def prepare(self):
         self.clear_items()
         max_pages = self.source.get_max_pages()
-        self.add_item(self.close_button)
-        self.add_item(self.queue_disconnect)
+        is_dj = await is_dj_logic(self.ctx)
+        if not self.is_history:
+            self.add_item(self.close_button)
+            self.add_item(self.queue_disconnect)
+            self.add_item(self.clear_queue_button)
 
         self.add_item(self.first_button)
         self.add_item(self.backward_button)
@@ -199,9 +212,12 @@ class QueueMenu(BaseMenu):
         self.add_item(self.last_button)
         self.add_item(self.refresh_button)
 
-        self.repeat_button_on.disabled = False
-        self.repeat_button_off.disabled = False
-        self.repeat_queue_button_on.disabled = False
+        if not self.is_history:
+            self.repeat_button_on.disabled = False
+            self.repeat_button_off.disabled = False
+            self.repeat_queue_button_on.disabled = False
+            self.clear_queue_button.disabled = False
+            self.show_history_button.disabled = False
 
         self.forward_button.disabled = False
         self.backward_button.disabled = False
@@ -209,6 +225,17 @@ class QueueMenu(BaseMenu):
         self.last_button.disabled = False
         self.refresh_button.disabled = False
 
+        if max_pages == 2:
+            self.first_button.disabled = True
+            self.last_button.disabled = True
+        elif max_pages == 1:
+            self.forward_button.disabled = True
+            self.backward_button.disabled = True
+            self.first_button.disabled = True
+            self.last_button.disabled = True
+        if self.is_history:
+            return
+
         self.previous_track_button.disabled = False
         self.paused_button.disabled = False
         self.resume_button.disabled = False
@@ -226,15 +253,8 @@ class QueueMenu(BaseMenu):
         self.add_item(self.previous_track_button)
         self.add_item(self.stop_button)
 
-        if max_pages == 2:
-            self.first_button.disabled = True
-            self.last_button.disabled = True
-        elif max_pages == 1:
-            self.forward_button.disabled = True
-            self.backward_button.disabled = True
-            self.first_button.disabled = True
-            self.last_button.disabled = True
-        if player := self.cog.lavalink.get_player(self.source.guild_id):
+        if (player := self.cog.lavalink.get_player(self.source.guild_id)) and is_dj is not False:
+            await player.config.update()
             if player.paused:
                 self.add_item(self.resume_button)
             else:
@@ -243,6 +263,7 @@ class QueueMenu(BaseMenu):
                 self.shuffle_button.disabled = True
                 self.remove_from_queue_button.disabled = True
                 self.play_now_button.disabled = True
+                self.clear_queue_button.disabled = True
             if not player.current:
                 self.equalize_button.disabled = True
                 self.stop_button.disabled = True
@@ -252,6 +273,9 @@ class QueueMenu(BaseMenu):
                 self.increase_volume_button.disabled = True
             if player.history.empty():
                 self.previous_track_button.disabled = True
+                self.show_history_button.disabled = True
+            else:
+                self.add_item(self.show_history_button)
             if player.config.repeat_current:
                 self.add_item(self.repeat_button_on)
             elif player.config.repeat_queue:
@@ -268,6 +292,7 @@ class QueueMenu(BaseMenu):
             self.skip_button.disabled = True
             self.previous_track_button.disabled = True
             self.repeat_button_off.disabled = True
+            self.show_history_button.disabled = True
             self.shuffle_button.disabled = True
             self.decrease_volume_button.disabled = True
             self.increase_volume_button.disabled = True
@@ -278,6 +303,7 @@ class QueueMenu(BaseMenu):
             self.remove_from_queue_button.disabled = True
             self.play_now_button.disabled = True
             self.repeat_queue_button_on.disabled = True
+            self.clear_queue_button.disabled = True
 
             self.add_item(self.resume_button)
             self.add_item(self.repeat_button_off)
@@ -290,7 +316,8 @@ class QueueMenu(BaseMenu):
         self.add_item(self.remove_from_queue_button)
         self.add_item(self.play_now_button)
         self.equalize_button.disabled = True
-        self.equalize_button.disabled = True
+        if is_dj is False:
+            self.queue_disconnect.disabled = True
 
     @property
     def source(self) -> QueueSource:
